# -*- coding: utf-8 -*-

import logging
import json
import datetime
import csv
import cStringIO as StringIO

from django import http
from django.core import serializers
from django.db import transaction
from django.shortcuts import redirect
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404
from snf_django.lib.api.utils import isoformat

from snf_django.lib import api
from synnefo.lib.services import get_path

from astakos.im import settings as astakos_settings
from astakos.im.views.im import require_http_methods, cookie_fix, \
    valid_astakos_user_required, get_context
from astakos.im.views.util import render_response

from survey import settings
from survey.models import Survey
from survey.forms import SurveySubmitForm
from survey import messages as survey_messages
from survey import util

logger = logging.getLogger(__name__)

PERMITTED_GROUPS = settings.PERMITTED_GROUPS
try:
    AUTH_URL = get_path(astakos_settings.astakos_services,
                        "astakos_identity.endpoints")[0]["publicURL"]
except (IndexError, KeyError) as e:
    logger.error("Failed to load Astakos Auth URL: %s", e)
    AUTH_URL = None


@require_http_methods(["GET"])
@cookie_fix
@valid_astakos_user_required
def index(request):
    try:
        survey = Survey.objects.active().order_by('-created_at')[0]
    except IndexError:
        messages.warning(request, survey_messages.NO_ACTIVE_SURVEY)
        return redirect(reverse('index'))
    return redirect(reverse('survey', kwargs={'sid': survey.sid}))

def survey(request, sid, lang=settings.LANGUAGE):
    template_name = 'survey/survey_form.html'
    survey = Survey.objects.active().get(sid=sid)
    lang = request.REQUEST.get('lang', lang)
    if lang not in survey.languages:
        lang = survey.languages[0]

    survey.set_i18n(lang)
    survey_form = SurveySubmitForm.from_instance(survey)


    if request.method == 'POST':
        survey_form = SurveySubmitForm.from_instance(survey, request.POST)
        if survey_form.is_valid():
            with transaction.commit_on_success():
                submission = survey_form.save(request.user,
                                              request.META.get('REMOTE_ADDR'))
            messages.success(request, survey_messages.SUBMIT_SUCCESS)
            return redirect(reverse('index'))
        else:
            messages.warning(request, survey_messages.SUBMIT_ERROR)

    extra_context = {}
    return render_response(template_name,
                           survey_form=survey_form,
                           survey=survey,
                           lang=lang,
                           lang_map=survey_messages.LANG_LINK_MAP,
                           languages=survey.languages,
                           context_instance=get_context(request,
                                                        extra_context))


def _dthandler(obj):
    if isinstance(obj, datetime.datetime):
        return isoformat(obj)
    else:
        raise TypeError


@api.api_method(http_method='GET', user_required=True, token_required=True,
                astakos_auth_url=AUTH_URL,
                logger=logger, serializations=['json'])
@api.user_in_groups(permitted_groups=PERMITTED_GROUPS,
                    logger=logger)
def results(request, sid):
    survey = get_object_or_404(Survey, sid=sid)
    lang = request.GET.get("lang", settings.LANGUAGE)
    if not lang in survey.languages:
        lang = survey.languages[0]

    questions = survey.questions.active()
    qlist = ['date', 'ip', 'user']
    def qtitle(q):
        q.set_i18n(lang)
        return q.title
    qlist.extend(map(qtitle, questions))

    buffer = StringIO.StringIO()
    writer = util.UnicodeWriter(buffer, delimiter=",",
                                quotechar="\"", quoting=csv.QUOTE_MINIMAL)
    writer.writerow(qlist)

    for submission in survey.submissions.distinct():
        alist = [unicode(submission.created_at), submission.ip_address or '',
                 submission.user.log_display]
        for answer in submission.answers.active().select_related():
            q = answer.question
            q.set_i18n(lang)
            a = q.module.answer_display(answer.answer)
            alist.append(a)
        writer.writerow(alist)
    buffer.seek(0)
    return http.HttpResponse(buffer.read(), status=200, content_type='text/plain')
