import json

from django.db import models
from django.db import transaction

from picklefield.fields import PickledObjectField
from survey.types import *

# I18N helpers
class I18NModelProxy(models.Model):
    i18n_model = None

    def load_i18n(self, lang):
        self._lang = lang
        i18n = self.i18n.get(language=lang)
        fields = self.i18n.model._meta.fields
        for field in fields:
            if field.name in ['id', 'language']:
                continue
            setattr(self, field.name, getattr(i18n, field.name))

    def set_i18n(self, lang, **kwargs):
        create = kwargs.pop('create', False)
        try:
            i18n = self.i18n.get(language=lang)
            for key, value in kwargs.iteritems():
                setattr(i18n, key, value)
        except:
            if create:
                i18n = self.i18n.create(language=lang, **kwargs)
            else:
                raise
        i18n.save()
        self.load_i18n(lang)

    class Meta:
        abstract = True


class I18NModel(models.Model):
    language = models.CharField(max_length=6, null=False, blank=False)
    parent = None

    class Meta:
        abstract = True
        unique_together = (('language', 'parent'),)


class SurveyI18N(I18NModel):
    parent = models.ForeignKey('Survey', related_name='i18n')
    title = models.TextField()
    description = models.TextField(null=True, default=None)


class SurveyManager(models.Manager):

    def active(self):
        return self.filter(is_active=True)

    @transaction.commit_on_success
    def load(self, data):
        survey, created = Survey.objects.get_or_create(sid=data['id'])
        languages = data['languages']
        survey.languages = languages
        survey.is_active = data['is_active']
        survey.save()
        for lang in languages:
            title = data['title'][lang]
            description = ""
            if data.get('description', None):
                description = data['description'][lang]
            survey.set_i18n(lang, create=True,
                            title=title, description=description)

        for qid, question in data['questions'].iteritems():
            q, created = survey.questions.get_or_create(qid=qid)
            if not created and q.type != question['type']:
                raise Exception("Question type cannot be chaged")

            q.type = question['type']
            q.required = question['required']
            q.params = q.module.clean_params(question.get('params', {}))
            q.order = question['order']
            q.disabled = question.get('disabled', False)
            qdata = q.module.clean_data(question.get('data', {}))

            for lang in languages:
                title = question['title'][lang]
                help_text = question.get('help_text')
                if help_text:
                    help_text = help_text[lang]
                if qdata:
                    data = qdata.get(lang)
                q.set_i18n(lang, create=True,
                           data=data, title=title, help_text=help_text)

            q.save()
        return survey, created


class Survey(I18NModelProxy):
    """
    Survey model
    """
    sid = models.CharField(max_length=40, unique=True, blank=False,
                           null=False)
    is_active = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    languages = PickledObjectField(null=True)

    objects = SurveyManager()

    def params(self, survey_details=False):
        sdata = {}
        if survey_details:
            sdata = {
                'created_at': self.created_at,
                'updated_at': self.updated_at,
                'languages': self.languages
            }
            sdata['i18n'] = {}
            for lang in self.languages:
                try:
                    o = self.i18n.get(language=lang)
                except:
                    continue
                sdata['i18n'][lang] =  {
                    'title': o.title,
                    'description': o.description
                }

            sdata['questions'] = []
            for q in self.questions.filter().order_by('order'):
                qdata = {
                    'id': q.qid,
                    'requred': q.required,
                    'type': q.type,
                    'order': q.order,
                    'params': q.params,
                }
                qdata['i18n'] = {}
                for lang in self.languages:
                    try:
                        o = q.i18n.get(language=lang)
                    except:
                        pass
                    qdata['i18n'][lang] = {
                        'title': o.title,
                        'help_text': o.help_text,
                        'data': o.data
                    }

                sdata['questions'].append(qdata)

        return sdata



class QuestionI18N(I18NModel):
    parent = models.ForeignKey('Question', related_name='i18n')
    title = models.TextField()
    help_text = models.TextField(null=True, default=None)
    data = PickledObjectField(null=True)


class QuestionManager(models.Manager):
    use_for_related_fields = True

    def active(self):
        return self.filter(disabled=False).order_by('order')


class Question(I18NModelProxy):
    qid = models.CharField(max_length=40, blank=False, null=False)
    survey = models.ForeignKey(Survey, related_name='questions')
    required = models.BooleanField(default=False)
    type = models.CharField(max_length=20, choices=QUESTION_TYPE_CHOICES)
    order = models.PositiveIntegerField(default=0)
    params = PickledObjectField()
    disabled = models.BooleanField(default=False)

    objects = QuestionManager()

    class Meta:
        ordering = ('order', )

    @property
    def module(self):
        return QUESTION_TYPES.get(self.type)(self)

    class Meta:
        unique_together = (('survey', 'qid'),)


class SubmissionManager(models.Manager):

    def distinct(self):
        return self.filter().distinct('user').order_by('user', '-created_at')


class Submission(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    user = models.ForeignKey('im.AstakosUser')
    ip_address = models.CharField(max_length=255, null=True, blank=True,
                                  default=None)
    survey = models.ForeignKey(Survey, related_name="submissions")
    objects = SubmissionManager()


class AnswerManager(models.Manager):
    use_for_related_fields = True
    def active(self):
        return self.filter(question__disabled=False).order_by('question__order')


class Answer(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    submission = models.ForeignKey('Submission', related_name='answers')
    question = models.ForeignKey(Question)
    answer = PickledObjectField()

    objects = AnswerManager()

    class Meta:
        ordering = ('-created_at', )
